/*

*/

import framebuffer.FrameBuffer;

import java.util.Random;
import java.awt.Color;
import java.io.FileInputStream;
import java.io.IOException;

/**
   This program writes pixel data to a PPM file using the FrameBuffer class.
   The pixel data is a sequence of red, then green, then blue pixels.

   In addition, this example creates two "viewports" within the framebuffer.
   One viewport holds random pixels and the other viewport contains the Yoda picture.
*/
public class Example_3
{
   public static void main(String[] args)
   {
      String filename = "Example_3.ppm";

      int height = 600;
      int width  = 600;
      FrameBuffer fb = new FrameBuffer(width, height);

      for (int y = 0; y < height; ++y)
      {
         for(int x = 0; x < width-3; x+=3)
         {
            fb.setPixelFB(x+0, y, new Color(255, 0,     0) );  // a red pixel
            fb.setPixelFB(x+1, y, new Color(0,   255,   0) );  // a green pixel
            fb.setPixelFB(x+2, y, new Color(0,   0,   255) );  // a blue pixel
         }
      }

      // Create a "viewport" within the framebuffer to hold random pixels.
      FrameBuffer.Viewport vp = fb.new Viewport(64, 64, 256, 128); // upper-left-hand-corner, width, height

      Random random = new Random();
      for (int y = 0; y < 128; ++y)
      {
         for (int x = 0; x < 256; ++x)
         {
            int r = random.nextInt(256);
            int g = random.nextInt(256);
            int b = random.nextInt(256);
            vp.setPixelVP(x, y, new Color(r, g, b));
         }
      }

/*
      // Create a "viewport" within the framebuffer to hold the Yoda image.
      vp = fb.new Viewport(300, 300, "Yoda.p6.ppm"); // upper-left-hand-corner
*/
      // Create a "viewport" within the framebuffer to hold the raw Yoda image.
      vp = fb.new Viewport(300, 300, 256, 256); // upper-left-hand-corner, width, height

      // Read the pixel data in a RAW file.
      String inputFileName = "Yoda.raw";
      try
      {
         FileInputStream fis = new FileInputStream(inputFileName);

         // Create a data array.
         byte[] pixelData = new byte[3];
         // Read pixel data, one pixel at a time, from the RAW file.
         for (int y = 0; y < 256; ++y)
         {
            for (int x = 0; x < 256; ++x)
            {
               if ( fis.read(pixelData, 0, 3) != 3 )
               {
                  System.err.printf("ERROR! Could not load %s\n", inputFileName);
                  System.exit(-1);
               }
               int r = pixelData[0];
               int g = pixelData[1];
               int b = pixelData[2];
               if (r < 0) r = 256+r;  // convert from signed byte to unsigned byte
               if (g < 0) g = 256+g;
               if (b < 0) b = 256+b;
               vp.setPixelVP(x, y, new Color(r, g, b));
            }
         }

         fis.close();
      }
      catch (IOException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not read %s\n", inputFileName);
         System.exit(-1);
      }

      fb.dumpFB2File( filename );
   }
}
