/*

*/

import framebuffer.FrameBuffer;

import java.awt.Color;

/**
   This program takes the (square) image given on
   the command-line and creates a new image that
   is tiled by eight flipped and rotated copies
   of the original image.
*/
public class FlippedViewports
{
   public static void main(String[] args)
   {
      // Check for a file name on the command line.
      // The source image needs to be a square image.
      if ( 0 == args.length )
      {
         System.err.println("Usage: java FlippedViewports <square-PPM-file>");
         System.exit(-1);
      }
      FrameBuffer fb1 = new FrameBuffer( args[0] );

      // the output image
      int height = fb1.getHeightFB();
      int width  = fb1.getWidthFB();
      FrameBuffer fb2 = new FrameBuffer(4*width, 2*height);

      FrameBuffer.Viewport vp1 = fb2.new Viewport(0*width, 0*height, fb1);
      FrameBuffer.Viewport vp2 = fb2.new Viewport(1*width, 0*height, width, height);
      FrameBuffer.Viewport vp3 = fb2.new Viewport(2*width, 0*height, width, height);
      FrameBuffer.Viewport vp4 = fb2.new Viewport(3*width, 0*height, width, height);
      FrameBuffer.Viewport vp5 = fb2.new Viewport(0*width, 1*height, width, height);
      FrameBuffer.Viewport vp6 = fb2.new Viewport(1*width, 1*height, width, height);
      FrameBuffer.Viewport vp7 = fb2.new Viewport(2*width, 1*height, width, height);
      FrameBuffer.Viewport vp8 = fb2.new Viewport(3*width, 1*height, width, height);

      flipR(vp1, vp2);
      flipR(vp2, vp3);
      flipR(vp3, vp4);

      flipV(vp1, vp5);
      flipR(vp5, vp6);
      flipR(vp6, vp7);
      flipR(vp7, vp8);

      String baseName = args[0].substring(0, args[0].lastIndexOf("."));
      String savedFileName = baseName + "_FlippedViewports" + ".ppm";
      fb2.dumpFB2File( savedFileName );
   }


   // Rotate the contents of vp1 90 degrees clockwise into vp2.
   // Assume that the two viewports have the same dimensions.
   public static void flipR(FrameBuffer.Viewport vp1, FrameBuffer.Viewport vp2)
   {
      final int width = vp1.getWidthVP();
      final int height = vp2.getHeightVP();
      for (int y = 0; y < height; ++y)
      {
         for(int x = 0; x < width; ++x)
         {
            Color c = vp1.getPixelVP(x, y);
            vp2.setPixelVP(width - 1 - y, x, c);
         }
      }
   }

   // Rotate the contents of vp1 90 degrees clockwise into vp2.
   // Assume that the two viewports have the same dimensions.
   public static void flipV(FrameBuffer.Viewport vp1, FrameBuffer.Viewport vp2)
   {
      final int width = vp1.getWidthVP();
      final int height = vp2.getHeightVP();
      for (int y = 0; y < height; ++y)
      {
         for(int x = 0; x < width; ++x)
         {
            Color c = vp1.getPixelVP(x, y);
            vp2.setPixelVP(x, height - 1 - y, c);
         }
      }
   }
}
